//  AdSuiteViewController.m

#define kAdHeight 50.0
#define kAdWidth 320.0

#import "AdSuiteViewController.h"
#import "ASIHTTPRequest.h"
#import "JSONKit.h"
#import "AdSuite.h"

@implementation AdSuiteViewController
@synthesize iAdView;
@synthesize iAdShow;
@synthesize adMobBannerView;
@synthesize adOrderList;
@synthesize delegate=_delegate;

#pragma mark - iAd
// iAd 광고 추가 
- (void)createiAdBannerView {
    CGRect frame;
    NSString *adBannerSize = ADBannerContentSizeIdentifierPortrait;
    frame.size = [ADBannerView sizeFromBannerContentSizeIdentifier:adBannerSize];
    
    frame.origin = CGPointMake(0.0, kAdHeight);
    
    ADBannerView *adViewI = [[ADBannerView alloc] initWithFrame:frame];
    [adViewI setRequiredContentSizeIdentifiers:[NSSet setWithObject:adBannerSize]];
    [adViewI setCurrentContentSizeIdentifier:adBannerSize];
    [adViewI setDelegate:self];
    [adViewI setAutoresizesSubviews:YES];
    [self.view addSubview:adViewI];
    self.iAdView = adViewI;
    [adViewI release];
}

- (void)bannerViewDidLoadAd:(ADBannerView *)banner {
    // iAd 로딩
    if (!self.iAdShow) {
        [UIView beginAnimations:@"animateAdBannerOn" context:NULL];
        banner.frame = CGRectOffset(banner.frame, 0, -kAdHeight);
        [UIView commitAnimations];
        self.iAdShow = YES;
    }
    [self reOrderAdView];
}

- (void)bannerView:(ADBannerView *)banner didFailToReceiveAdWithError:(NSError *)error {
    // iAd 로딩 실패
    if (self.iAdShow) {
        [UIView beginAnimations:@"animateAdBannerOff" context:NULL];
        banner.frame = CGRectOffset(banner.frame, 0, kAdHeight);
        [UIView commitAnimations];
        self.iAdShow = NO;
    }
}

- (BOOL)bannerViewActionShouldBegin:(ADBannerView *)banner willLeaveApplication:(BOOL)willLeave {
    // iAd 액션 시작
    self.view.hidden = YES;
    return YES;
}

- (void)bannerViewActionDidFinish:(ADBannerView *)banner {
    // iAd 액션 종료
    [self.delegate setAdSuiteFrame:self.view];
    self.view.hidden = NO;
}

#pragma mark - AdMob 
- (void)createAdMobView {
    // AdMob 추가
    self.adMobBannerView = [[GADBannerView alloc] 
                            initWithFrame:CGRectMake(0.0,
                                                     0.0,
                                                     GAD_SIZE_320x50.width,
                                                     GAD_SIZE_320x50.height)];
    self.adMobBannerView.adUnitID = ADMOB_PUBLISHER_ID;
    self.adMobBannerView.rootViewController = [self.delegate adSuiteRootViewController:self.view];
    self.adMobBannerView.delegate = self;
    [self.view addSubview:self.adMobBannerView];
    [self.adMobBannerView release];
    GADRequest *request = [GADRequest request];
    request.testDevices = [NSArray arrayWithObjects:
                           GAD_SIMULATOR_ID,                                // Simulator
                           @"xxxxxxxxxxxxxxxxxxxxxxxxx",     // 본인의 아이폰 UDID 입력
                           nil];
    [self.adMobBannerView loadRequest:request];
}

- (void)adViewDidReceiveAd:(GADBannerView *)bannerView {
    // AdMob 광고 받음
    self.adMobBannerView.hidden = NO;
    [self reOrderAdView];
}

- (void)adView:(GADBannerView *)bannerView didFailToReceiveAdWithError:(GADRequestError *)error {
    // AdMob 광고 받기 실패
    self.adMobBannerView.hidden = YES;
}

- (void)adViewWillPresentScreen:(GADBannerView *)bannerView {
    // AdMob 광고를 클릭
}

- (void)adViewWillDismissScreen:(GADBannerView *)bannerView {
    // AdMob 광고화면 닫기 클릭
}

- (void)adViewDidDismissScreen:(GADBannerView *)bannerView {
    // AdMob 광고화면 닫기 클릭
}

- (void)adViewWillLeaveApplication:(GADBannerView *)bannerView {
    // AdMob 광고에서 다른앱으로 전환
}

#pragma mark - 광고제어 
// 광고 제어 URL에 접속하여 제어값을 가져온다.
- (void)startGetAdAdjust {
    ASIHTTPRequest *request = [ASIHTTPRequest requestWithURL:[NSURL URLWithString:AD_ADJUST_URL]];
    request.delegate = self;
    [request setDidFinishSelector:@selector(finishGetAdAdjust:)];
    [request setDidFailSelector:@selector(failsGetAdAdjust:)];
    [request startAsynchronous];
}

// 광고 제어 URL을 성공적으로 가져오는 경우 실행
- (void)finishGetAdAdjust:(ASIHTTPRequest *)request {
    NSDictionary *json = [[request responseString] objectFromJSONString];
    if (json != nil) {
        NSArray *_adOrder = [json objectForKey:@"order"];
        self.adOrderList = (_adOrder != nil && [_adOrder count] > 0) ? _adOrder : self.adOrderList;
        BOOL _iAdOn = [[json objectForKey:AD_TYPE_IAD] boolValue];
        BOOL _adMobOn = [[json objectForKey:AD_TYPE_ADMOB] boolValue];
        NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
        [prefs setObject:self.adOrderList forKey:USER_DIC_AD_ORDER];
        [prefs setBool:_iAdOn forKey:USER_DIC_IAD_ON];
        [prefs setBool:_adMobOn forKey:USER_DIC_ADMOB_ON];
        [prefs synchronize];
        
        [self addUsedAdView:_iAdOn adMobOn:_adMobOn];
        [self removeUnUsedAdView:_iAdOn adMobOn:_adMobOn];
    } else {
        self.adOrderList = nil;
        self.adOrderList = [NSArray arrayWithObjects:AD_TYPE_IAD, AD_TYPE_ADMOB, nil];
        [self addUsedAdView:YES adMobOn:YES];
        [self removeUnUsedAdView:YES adMobOn:YES];
    }
}

// 광고제어 URL 가져오기에 실패하는 경우 실행
- (void)failsGetAdAdjust:(ASIHTTPRequest *)request {
    self.adOrderList = nil;
    self.adOrderList = [NSArray arrayWithObjects:AD_TYPE_IAD, AD_TYPE_ADMOB, nil];
    [self addUsedAdView:YES adMobOn:YES];
    [self removeUnUsedAdView:YES adMobOn:YES];
}

// 광고뷰의 순서를 설정값에 맞게 재배치
- (void)reOrderAdView {    
    NSEnumerator *enumerator = [self.adOrderList reverseObjectEnumerator];
    for (NSString *adType in enumerator) {
        if ([adType isEqualToString:AD_TYPE_ADMOB] && self.adMobBannerView != nil) {
            [self.view bringSubviewToFront:self.adMobBannerView];
        } else if ([adType isEqualToString:AD_TYPE_IAD] && self.iAdView != nil) {
            [self.view bringSubviewToFront:self.iAdView];
        }
    }
}

// 현재 설정값에 보여져야 하는데 안보여지고 있는 광고 뷰 추가
- (void)addUsedAdView:(BOOL)_iAdOn adMobOn:(BOOL)_adMobOn {
    if (self.iAdView == nil && _iAdOn) {
        // iAd 광고뷰는 비어있는데 iAd 노출값은 On 인경우 광고 추가
        [self createiAdBannerView];
    }
    if (self.adMobBannerView == nil && _adMobOn) {
        // adMob 광고뷰는 비어있는데 adMob 노출값은 On 인경우 광고 추가
        [self createAdMobView];
    }
}

// 사용하지 않는다고 설정한 광고뷰는 화면에서 제거
- (void)removeUnUsedAdView:(BOOL)_iAdOn adMobOn:(BOOL)_adMobOn {
    NSArray *subviews = self.view.subviews;
    for (UIView *subview in subviews) {
        if (subview == self.iAdView && !_iAdOn) {
            // 사용하지 않는다고 설정한 iAd 제거
            [subview removeFromSuperview];
        } else if (subview == self.adMobBannerView && !_adMobOn) {
            // 사용하지 않는다고 설정한 AdMob 제거
            [subview removeFromSuperview];
        }
    }
}

#pragma mark - View lifecycle
- (void)viewDidLoad {
    [super viewDidLoad];
    // 광고 위치 설정 
    [self.delegate setAdSuiteFrame:self.view];
    // 광고 설정 가져오기
    NSUserDefaults *userDic = [NSUserDefaults standardUserDefaults];
    NSArray *prefAdOrderList = [userDic objectForKey:USER_DIC_AD_ORDER];
    self.adOrderList = (prefAdOrderList != nil && [prefAdOrderList count] > 0) ? prefAdOrderList : [NSArray arrayWithObjects:AD_TYPE_IAD, AD_TYPE_ADMOB, nil];
    BOOL _iAdOn = [userDic objectForKey:USER_DIC_IAD_ON] == nil ? YES : [userDic boolForKey:USER_DIC_IAD_ON];
    BOOL _adMobOn = [userDic objectForKey:USER_DIC_ADMOB_ON] == nil ? YES : [userDic boolForKey:USER_DIC_ADMOB_ON];
    // 광고제어 값 가져오기
    [self startGetAdAdjust];
    // iAd, AdMob 광고를 뷰에 추가 
    [self addUsedAdView:_iAdOn adMobOn:_adMobOn];
}

- (void)viewDidUnload {
    self.iAdView = nil;
    self.adMobBannerView = nil;
    self.adOrderList = nil;
    [super viewDidUnload];
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)dealloc {
    self.iAdView.delegate = nil;
    if (self.iAdView != nil) {
        [self.iAdView release];
    }
    self.adMobBannerView.delegate = nil;
    if (self.adMobBannerView != nil) {
        [self.adMobBannerView release];
    }
    if (self.adOrderList != nil) {
        [self.adOrderList release];
    }
    [super dealloc];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

@end