// ViewController.m

#import "ViewController.h"
#import "FourImageView.h"

static NSUInteger kNumberOfPages = 3;

@implementation ViewController
@synthesize fourImageViewList;
@synthesize imageList;
@synthesize pageControl;
@synthesize imageScrollView;

#pragma mark - Scrollview
// 파라미터로 입력받은 페이지에 해당하는 커스텀뷰를 생성하고 메인뷰에 추가.
- (void)loadScrollViewWithPage:(int)page {
    if (page < 0)
        return;
    if (page >= kNumberOfPages)
        return;
    
    FourImageView *fourImageView = [fourImageViewList objectAtIndex:page];
    if ((NSNull *)fourImageView == [NSNull null]) {
        // 주어진 페이지의 이미지 정보 얻기 
        NSArray *pageInfo = [self.imageList objectAtIndex:page];
        NSString *imageName1 = [pageInfo objectAtIndex:0];
        NSString *imageName2 = [pageInfo objectAtIndex:1];
        NSString *imageName3 = [pageInfo objectAtIndex:2];
        NSString *imageName4 = [pageInfo objectAtIndex:3];
        
        // 뷰 컨트롤러 추가
        fourImageView = [[FourImageView alloc] init];
        fourImageView.imageName1 = imageName1;
        fourImageView.imageName2 = imageName2;
        fourImageView.imageName3 = imageName3;
        fourImageView.imageName4 = imageName4;
        [fourImageViewList replaceObjectAtIndex:page withObject:fourImageView];
        [fourImageView release];
    }
    
    // 스크롤뷰에 추가되어 있지 않을경우 추가 
    if (fourImageView.superview == nil) {
        CGRect frame = imageScrollView.frame;
        frame.origin.x = frame.size.width * page;
        frame.origin.y = 0;
        fourImageView.frame = frame;
        [imageScrollView addSubview:fourImageView];
    }
}

// 이미지뷰 목록을 초기화 하고 페이지 컨트롤을 리셋
- (void)resetFourImageViewList {
    if (fourImageViewList == nil) {
        fourImageViewList = [[NSMutableArray alloc] init];
    } else {
        [fourImageViewList removeAllObjects];
        for (UIView *subView in imageScrollView.subviews) {
            [subView removeFromSuperview];
        }
    }
    
    for (unsigned i = 0; i < kNumberOfPages; i++) {
        [fourImageViewList addObject:[NSNull null]];
    }
    
	pageControl.numberOfPages = kNumberOfPages;
    pageControl.currentPage = 0;
}

// 이미지 스크롤뷰 전체를 새로고침
- (void)refreshImageScrollView {
    NSUserDefaults *userDic = [NSUserDefaults standardUserDefaults];
    NSInteger pageCount = [[userDic objectForKey:@"page_count"] integerValue];
    if (pageCount > 0) {
        kNumberOfPages = pageCount;
    }
	[self resetFourImageViewList];
	[self loadScrollViewWithPage:0]; // 첫페이지 로딩
    // 이미지목록에 맞추어 스크롤뷰의 사이즈 재조정
	imageScrollView.contentSize = CGSizeMake(imageScrollView.frame.size.width * kNumberOfPages, imageScrollView.frame.size.height);
	[imageScrollView setContentOffset:CGPointMake(0, 0) animated:YES];	// 첫페이지로 스크롤
}

#pragma mark - scrollview delegate
// UIScrollViewDelegate 프로토콜 메서드로 스크롤이 완료될때마다 호출됨
- (void)scrollViewDidScroll:(UIScrollView *)sender {
    // 보여지는 화면의 50% 이상 차지하고 있는 쪽으로 페이지 스크롤
    CGFloat pageWidth = imageScrollView.frame.size.width;
    int page = floor((imageScrollView.contentOffset.x - pageWidth / 2) / pageWidth) + 1;
    pageControl.currentPage = page;
    
    [self loadScrollViewWithPage:page - 1];
    [self loadScrollViewWithPage:page];
    [self loadScrollViewWithPage:page + 1];
    
    // 화면에 보이지 않는 페이지는 제거
    for (int i = 0; i < [fourImageViewList count]; i++) {
        if (page == i || page - 1 == i || page + 1 == i) {
            continue;
        }
        FourImageView *fourImageView = [fourImageViewList objectAtIndex:i];
        if ((NSNull *)fourImageView == [NSNull null]) {
            continue;
        }
        [fourImageView removeFromSuperview];
        [fourImageViewList replaceObjectAtIndex:i withObject:[NSNull null]];
    }
}

#pragma mark - View lifecycle
- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Release any cached data, images, etc that aren't in use.
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // 이미지 파일 목록 읽기
    NSString *plistPath = [[NSBundle mainBundle] pathForResource:@"ImageList" ofType:@"plist"];
    NSDictionary *dic = [NSDictionary dictionaryWithContentsOfFile:plistPath];
    self.imageList = [dic objectForKey:@"imageList"];

    // 페이지 갯수 설정하기
    kNumberOfPages = [self.imageList count];
    
    // 스크롤뷰 초기 설정
    imageScrollView.delegate = self; // UIScrollViewDelegate 프로토콜을 구현한 객체로 지정
    imageScrollView.pagingEnabled = YES;
    imageScrollView.contentSize = CGSizeMake(imageScrollView.frame.size.width * kNumberOfPages, imageScrollView.frame.size.height);
    imageScrollView.showsHorizontalScrollIndicator = NO;
    imageScrollView.showsVerticalScrollIndicator = NO;
    imageScrollView.scrollsToTop = NO;
    [self refreshImageScrollView];  // 스크롤뷰 새로 고침
}

- (void)viewDidUnload
{
    [self setImageScrollView:nil];
    [self setPageControl:nil];
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
}

- (void)dealloc {
    [imageScrollView release];
    [pageControl release];
    [super dealloc];
}
@end
