//  MainViewController.m

#define HERE_STRING @"@"

#import "MainViewController.h"
#import "ASIHTTPRequest.h"
#import "Reachability.h"
#import "TFHpple.h"
#import "JSONKit.h"

static NSString *YAHOO_APP_ID = @"xxxxx"; // 야후 API 키값 입력

@implementation MainViewController
@synthesize alertButton;
@synthesize alertTableView;
@synthesize reportDic;
@synthesize locationNameDic;
@synthesize isBackgroundTrace;
@synthesize locationManager;
@synthesize curLocation;

#pragma mark - Util
// 공백, 개행문자등을 제거
- (NSString *)trimString:(NSString *)string {
	NSString *result = [string stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
	return result;
}

// 문자열을 사전형 데이터로 재가공
- (NSMutableDictionary *)getReportDic:(NSString *)rawString {
	// 경상북도(영천시 // 경산시 // 칠곡군 // 포항시 // 경주시) // 대구광역시
	// o 폭염경보 : 경상북도(영천시, 경산시, 칠곡군, 포항시, 경주시), 대구광역시
	// o 호우경보 : 제주도(제주도산간)
	// o 폭염주의보 : 경상북도(구미시, 군위군, 청도군, 고령군, 성주군, 김천시, 안동시, 의성군, 청송군, 영덕군)
	// o 호우주의보 : 경기도(김포시, 동두천시, 연천군, 포천시, 가평군, 고양시, 양주시, 의정부시, 파주시, 구리시, 남양주시, 양평군), 강원도(철원군, 화천군), 충청남도(태안군), 경상남도(의령군, 진주시, 산청군), 서울특별시, 인천광역시 
	// <참고사항>o 없음
	/*
	 section 
     [string, string...] 경보이름 목록
	 row
     [
     {region, subregion},  지역이름
     {} ...]
	 reference - 참고사항 뒷부분 문자열
	 */
	NSArray *rawArray = [rawString componentsSeparatedByString:@"<참고사항>"];
	if ([rawArray count] <= 0) {
		return nil;
	}
	
	NSMutableArray *section = [NSMutableArray array];
	NSMutableArray *row = [NSMutableArray array];
	
	NSString *reportRawString = [rawArray objectAtIndex:0];
	NSString *referenceString = [rawArray objectAtIndex:1];
	NSArray *reportList = [reportRawString componentsSeparatedByString:@"o"];
	for (NSString *item in reportList) { // 기상특보 별로 반복 실행
		item = [item stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
		if ([item length] == 0) {
			continue;
		}
        
		// 경보 이름 얻어내기 
		NSString *alertName;
		NSScanner *theScanner = [NSScanner scannerWithString:item];
		[theScanner scanUpToString:@":" intoString:&alertName];
		[theScanner scanString:@":" intoString:nil];
		[theScanner scanUpToString:@"" intoString:&item];
		
		// 경보 지역 알아내기 
		NSMutableArray *rowArray = [NSMutableArray array];
		NSMutableDictionary *rowDic = [NSMutableDictionary dictionary];
		NSArray *regionList = [item componentsSeparatedByString:@","];
		NSInteger subListCount = 0;
		NSMutableArray *subRegionNameList = [NSMutableArray array];
		for (NSString *region in regionList) { // 지역이름이 여러개일경우 같은 반복 실행
			NSString *rootRegionName;
			NSRange subRegionNameStartRange = [region rangeOfString:@"("];
			NSRange subRegionNameEndRange = [region rangeOfString:@")"];
			
			if (subListCount > 0) { // 하위 지역을 가지고 있는경우 
				if (subRegionNameStartRange.length > 0) { // 하위지역 내부에 또 하위지역을 가지는경우 
					[subRegionNameList addObject:[self trimString:region]];
					continue;
				} else if (subRegionNameEndRange.length > 0) { // 하위지역 리스트 종료
					region = [region stringByTrimmingCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@")"]];
					[subRegionNameList addObject:[self trimString:region]];
					subListCount--;
				}else { // 하위지역 리스트 추가중 
					[subRegionNameList addObject:[self trimString:region]];
					continue;
				}
			}else {
				if (subRegionNameStartRange.length > 0) { // 하위지역 리스트 추가 시작 (상위 지역 이름과 하위 지역이름 추출)
					NSArray *regionNameArray = [region componentsSeparatedByString:@"("];
					rootRegionName = [regionNameArray objectAtIndex:0];
					NSString *subRegionName = [regionNameArray objectAtIndex:1];
					subListCount++;
					
					if (subRegionNameEndRange.length > 0) { // 내부내역 리스트 곧바로 종료 
						subRegionName = [subRegionName stringByTrimmingCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@")"]];
						[subRegionNameList addObject:[self trimString:subRegionName]];
						subListCount--;
					}else {
						[subRegionNameList addObject:[self trimString:subRegionName]];
						continue;
					}
				} else { // 내부내역을 가지지 않은 광역시일 경우 이름 추가
					rootRegionName = [self trimString:region];
				}
			}
            
			[rowDic setValue:[self trimString:rootRegionName] forKey:@"region"];
			[rowDic setValue:subRegionNameList forKey:@"subregion"];
			subRegionNameList = [NSMutableArray array];
			[rowArray addObject:rowDic];
			rowDic = [NSMutableDictionary dictionary];
		}
		[section addObject:[self trimString:alertName]];
		[row addObject:rowArray];
	}
    // 최종 데이터 작업
	NSMutableDictionary *finalDic = [[[NSMutableDictionary alloc] init] autorelease];
	[finalDic setValue:section forKey:@"section"];
	[finalDic setValue:row forKey:@"row"];
	[finalDic setValue:referenceString forKey:@"reference"];
	return finalDic;
}

// 기상특보 알람 켬/끔 전환
- (void)toggleLocationMonitoring {
    if (self.isBackgroundTrace) {
        // 위치변경 모니터링 종료
        [self.locationManager stopMonitoringSignificantLocationChanges];
        self.isBackgroundTrace = NO;
        [self.alertButton setImage:[UIImage imageNamed:@"tower_white.png"] forState:UIControlStateNormal];
    }else {
        // 위치변경 모니터링 시작
        [self.locationManager startMonitoringSignificantLocationChanges];
        self.isBackgroundTrace = YES;
        [self.alertButton setImage:[UIImage imageNamed:@"tower_orange.png"] forState:UIControlStateNormal];
    }
}

// 주어진 배열의 주어진 인덱스를 맨앞으로 보낸다.
- (void)reorderArray:(NSMutableArray *) array atIndex:(NSUInteger) index {
    NSMutableArray *tempArray = [[NSMutableArray alloc] init];
    NSIndexSet *targetIndexSet = [[NSIndexSet alloc] initWithIndexesInRange:NSMakeRange(0, index)];
    [tempArray addObject:[array objectAtIndex:index]];
    [tempArray addObjectsFromArray:[array objectsAtIndexes:targetIndexSet]];
    [array replaceObjectsInRange:NSMakeRange(0, index + 1) withObjectsFromArray:tempArray];
}

- (void)isHereSpecialReport {
    NSString *state = [self.locationNameDic objectForKey:@"state"];
    NSString *county = [self.locationNameDic objectForKey:@"county"];
    NSString *town = [self.locationNameDic objectForKey:@"town"];

    /* 현재 위치에 기상특보가 있는지 확인 및 경보이름을 가져온다. */
    BOOL isHere = NO;
    NSArray *row = [reportDic objectForKey:@"row"];
    NSString *reportName = @"";
    NSUInteger sectionIndex = 0;
    BOOL breakLoop = NO;
    for (NSArray *rowInnerArray in row) {
        for (NSDictionary *rowDic in rowInnerArray){
            NSString *region = [rowDic objectForKey:@"region"];
            if (region != nil && [region hasPrefix:state]){
                isHere = YES;
                reportName = [[reportDic objectForKey:@"section"] objectAtIndex:sectionIndex];
                // 표에서 색을 표시하기 위해 지역이름뒤에 @ 표시를 붙여준다.
                NSString *newRegionName = [NSString stringWithFormat:@"%@%@", region, HERE_STRING];
                [rowDic setValue:newRegionName forKey:@"region"];
                breakLoop = YES;
                break;
            }	
        }
        if (breakLoop) {
            break;
        }
        sectionIndex++;
    }
    if (isHere) {
        // 현재위치에 경보가 존재하는 경우 local notification... 앱이 실행중인경우 alert 창이 뜬다. 
        UIApplication *application = [UIApplication sharedApplication];
        UILocalNotification *localNotif = [[UILocalNotification alloc] init];
        if (localNotif) {
            localNotif.alertBody = [NSString stringWithFormat:@"현재지역에 기상특보 확인 : %@ , %@ %@ %@", reportName, state, county, town];
            localNotif.alertAction = @"보기";
            localNotif.applicationIconBadgeNumber = 0;
            [application presentLocalNotificationNow:localNotif];
            [localNotif release];
        }
		// 기상특보 테이블 배열을 다시한다. 현재 위치에 발령된 특보를 위쪽에 보이도록
        if (sectionIndex > 0) { // 현재위치의 기상특보가 맨위에 없는경우
            NSMutableArray *row = [reportDic objectForKey:@"row"];			
            NSMutableArray *section = [reportDic objectForKey:@"section"];
            [self reorderArray:row atIndex:sectionIndex];
            [self reorderArray:section atIndex:sectionIndex];
        }
    }
    [self.alertTableView reloadData];

    if (!isBackgroundTrace) { // 자동위치 추적모드가 동작중이 아닐때 위치 정보를 가져오고 나서 종료 
        [self.locationManager stopUpdatingLocation];
    }
}

#pragma mark - 현재위치 이름 정보 가져오기
/* 야후 지도 API 를 이용해서 위도, 경도를 가지고 지역 이름을 받아온다.
 * http://kr.open.gugi.yahoo.com/document/geocooder.php
 * country(대한민국), state(경기도, 서울특별시), county(성남시, 광진구), town(정자동, 역삼동)
 */
- (void)startReceiveLocationName {
    // 위도 경도값 설정
	NSNumber *latitude = [NSNumber numberWithDouble:curLocation.coordinate.latitude];
	NSNumber *longitude = [NSNumber numberWithDouble:curLocation.coordinate.longitude];
    // 야후 API 호출 URL 생성
	NSString *urlString = [NSString stringWithFormat:@"http://kr.open.gugi.yahoo.com/service/rgc.php?appid=%@&latitude=%@&longitude=%@&output=json", YAHOO_APP_ID, latitude, longitude];
	ASIHTTPRequest *request = [ASIHTTPRequest requestWithURL:[NSURL URLWithString:urlString]];
	[request setDelegate:self];
	[request setDidFinishSelector:@selector(finishReceiveLocationName:)];
    // 비동기 방식으로 API 호출
	[request startAsynchronous];
}

// 지역이름 데이터 다 받았을경우 실행됨
- (void)finishReceiveLocationName:(ASIHTTPRequest *)request {    
    NSString *strResponse = [request responseString];
    // API 결과값을 사전형으로 변경
    NSMutableDictionary *resDic = [[[strResponse objectFromJSONString] objectForKey:@"ResultSet"] objectForKey:@"location"];
    NSMutableDictionary *nameDic = [NSMutableDictionary dictionaryWithDictionary:resDic];
    // state, county, town 값이 없을 경우 공백으로 변환. 
    if ([nameDic objectForKey:@"state"] == [NSNull null]) {
        [nameDic setValue:@"" forKey:@"state"];
    }
    if ([nameDic objectForKey:@"county"] == [NSNull null]) {
        [nameDic setValue:@"" forKey:@"county"];
    }
    if ([nameDic objectForKey:@"town"] == [NSNull null]) {
        [nameDic setValue:@"" forKey:@"town"];
    }
    self.locationNameDic = nameDic;
    // 현재 위치에 기상특보가 존재하는지 확인하는 메서드 실행 
    [self isHereSpecialReport];
}

#pragma mark - 기상특보 데이터 받아오기 
/* 기상청 홈페이지에서 현재 기상특보 데이터를 받아와서 사전 형태로 저장
 * http://www.kma.go.kr/weather/warning/status.jsp 현재 특보 현황 
 */
- (void)startReceiveSpecialReport {
	if ([[Reachability reachabilityForInternetConnection] currentReachabilityStatus] == NotReachable) {
		UIAlertView *alert = [[UIAlertView alloc] 
                              initWithTitle:@"알림" 
                              message:@"인터넷에 연결 할 수 없습니다." 
                              delegate:self 
                              cancelButtonTitle:@"닫기" 
                              otherButtonTitles:nil];
		[alert show];
		[alert release];
		self.reportDic = nil; // 날씨 경보 목록 비움
		[self.alertTableView reloadData]; // 테이블 데이터 새로고침
	} else {
        // 인터넷 연결이 가능한 경우 기상특보 정보 가져오기 시작
        NSURL *url = [NSURL URLWithString:@"http://www.kma.go.kr/weather/warning/status.jsp"];
        ASIHTTPRequest *request = [ASIHTTPRequest requestWithURL:url];
        [request setDelegate:self];
        [request setDidFinishSelector:@selector(finishReceiveSpecialReport:)];
        [request startAsynchronous]; // 비동기 방식으로 웹 데이터 가져오기 시작
    }
}

// 기상특보 데이터 다 받았을경우 실행 
- (void)finishReceiveSpecialReport:(ASIHTTPRequest *)request {
	NSString *data1		= [[request responseString] stringByReplacingOccurrencesOfString:@"<br />" withString:@""];
	NSData *data		= [data1 dataUsingEncoding:NSUnicodeStringEncoding];
	TFHpple *doc		= [[TFHpple alloc] initWithHTMLData:data];
	//NSArray *elements	= [doc search:@"//dl[@class='special_report_list2']/dd"];
    NSArray *elements	= [doc searchWithXPathQuery:@"//dl[@class='special_report_list2']/dd"];
	[elements retain];
	[doc release];
	TFHppleElement *element = [elements objectAtIndex:0];
	[elements release];
	NSString *result = [element content];
    // html 문자열에서 탭, 개행문자를 제거 
	result = [result stringByReplacingOccurrencesOfString:@"\t" withString:@""];
	result = [result stringByReplacingOccurrencesOfString:@"\n" withString:@""];
	result = [result stringByReplacingOccurrencesOfString:@"\r" withString:@""];
	// result 에는 아래와 같은 값이 들어가 있는 상태
	//result = @" o 폭염경보 : 경상북도(영천시, 경산시, 칠곡군, 포항시, 경주시), 대구광역시o 호우경보 : 서울특별시, 광주광역시o 폭염주의보 : 경상북도(구미시, 군위군, 청도군, 고령군, 성주군, 김천시, 안동시, 의성군, 청송군, 영덕군)o 호우주의보 : 경기도(김포시, 동두천시, 연천군, 포천시, 가평군, 고양시, 양주시, 의정부시, 파주시, 구리시, 남양주시, 양평군), 강원도(철원군, 화천군), 충청남도(태안군), 경상남도(의령군, 진주시, 산청군), 서울특별시, 인천광역시 <참고사항>o 없음";
    
    // 문자열을 사전형 데이터로 재가공
	self.reportDic = [self getReportDic:result];
	
	[self.alertTableView reloadData];
    [self startReceiveLocationName];
}

#pragma mark - 위치 변경시 호출 
- (void)locationManager:(CLLocationManager *)manager didUpdateToLocation:(CLLocation *)newLocation fromLocation:(CLLocation *)oldLocation {
    NSLog(@"위치 변경");
	if (!isBackgroundTrace) { // 자동위치 추적모드가 동작중이 아닐때는 위치 이동에 따라서 작동 안하게 
		return ;
	}
	
	NSDate* newDate = newLocation.timestamp;
	NSDate* oldDate = oldLocation.timestamp;
	
    NSTimeInterval howRecent = [newDate timeIntervalSinceDate:oldDate];
    if (abs(howRecent) > 20) { // 20초 이내에 변경된 경우에는 무시
		self.curLocation = newLocation;
		[self startReceiveSpecialReport]; // 기상특보 데이터 가져오기
	}
}

#pragma mark - UITableView
// 테이블 섹션의 갯수 지정
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
	return [[self.reportDic objectForKey:@"section"] count];
}

// 테이블 섹션별 항목수 지정
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	return [[[self.reportDic objectForKey:@"row"] objectAtIndex:section] count];
}

// 테이블의 특정 셀을 만들어서 제공
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    // 셀을 구분할 수 있는 고유값
	static NSString *CellIdentifier = @"WeatherCell";
    // 사용하지 않는 테이블셀이 있으면 재 사용 
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) { // 사용되지 않는 테이블셀이 없으면 생성 
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
    }
    
    // 날씨 경보 목록에서 현재 셀에 해당하는 데이터를 가져온다.
	NSArray *rowArray = [[reportDic objectForKey:@"row"] objectAtIndex:[indexPath section]];
	NSDictionary *rowDic = [rowArray objectAtIndex:[indexPath row]];
	NSString *region = [rowDic objectForKey:@"region"];
	// 지역이름의 끝이 @ 로 끝나는 경우 해당 지역에 기상특보가 있는것이므로 색이 있는 배경이미지를 사용 
	NSString *imageName = [NSString stringWithString:@"bg_box_white36"];
	if ([region hasSuffix:HERE_STRING]) {
		imageName = [NSString stringWithString:@"bg_box_orange36"];
	}
	UIImageView *imgView = [[UIImageView alloc] initWithImage:[UIImage imageNamed:imageName]];
	[imgView setAlpha:0.6]; // 투명도 60%
	cell.backgroundView = imgView; // 투명 이미지를 셀 배경으로 지정
	[imgView release];
	
    // 셀에 지역명을 설정한다.
    cell.textLabel.text = [region stringByReplacingOccurrencesOfString:HERE_STRING withString:@""];
    cell.textLabel.textAlignment = UITextAlignmentRight; // 오른쪽 정렬
	return cell;
}

// 테이블 섹션 헤더의 뷰 지정
- (UIView *) tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section {
    // 날씨 경보 목록에서 현재 섹션에 해당하는 경보 이름을 가져온다.
	NSString *alertName = [[reportDic objectForKey:@"section"] objectAtIndex:section];
	UIView* customView = [[UIView alloc] initWithFrame:CGRectMake(10.0, 0.0, 300.0, 44.0)];
    
	UILabel * headerLabel = [[UILabel alloc] initWithFrame:CGRectZero];
	headerLabel.backgroundColor = [UIColor clearColor];
	headerLabel.opaque = NO;
	headerLabel.textColor = [UIColor darkTextColor];
	headerLabel.highlightedTextColor = [UIColor whiteColor];
	headerLabel.font = [UIFont boldSystemFontOfSize:18];
	headerLabel.frame = CGRectMake(10.0, 0.0, 300.0, 44.0);
	headerLabel.text = alertName;
	headerLabel.alpha = 0.9;
	[customView addSubview:headerLabel];
	return customView;
}

// 테이블 셀의 높이 지정
- (CGFloat) tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {
	return 40.0;
}

#pragma mark - View lifecycle
- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Release any cached data, images, etc that aren't in use.
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // 테이블뷰의 배경 및 구분자 색상을 투명하게 처리
    self.alertTableView.backgroundColor = [UIColor clearColor];
	self.alertTableView.separatorColor = [UIColor clearColor];
    // 기상특보 변수 데이터 초기화
    reportDic = [[NSMutableDictionary alloc] init];
    // 위치정보 획득을 위한 초기화
    self.locationManager = [[CLLocationManager alloc] init];
	self.locationManager.delegate = self;
    [self toggleLocationMonitoring];
    // 현재위치 설정
	self.curLocation = self.locationManager.location;
    // 기상특보 데이터 받아오기 시작
    [self startReceiveSpecialReport];
}

- (void)viewDidUnload {
    [self.reportDic release];
    [self.locationNameDic release];
    [self setAlertButton:nil];
    [self setAlertTableView:nil];
    self.locationManager.delegate = nil;
    [self.locationManager release];
    [self.curLocation release];
    [super viewDidUnload];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
}

- (void)dealloc {
    [reportDic release];
    [locationNameDic release];
    [alertButton release];
    [alertTableView release];
    [locationManager release];
    [curLocation release];
    [super dealloc];
}

#pragma mark - Flipside View

- (void)flipsideViewControllerDidFinish:(FlipsideViewController *)controller
{
    [self dismissModalViewControllerAnimated:YES];
}

- (NSMutableDictionary *)flipsideViewControllerGetReportDic {
    return self.reportDic;
}

#pragma mark - IBAction
- (IBAction)showInfo:(id)sender
{        
    FlipsideViewController *controller = [[[FlipsideViewController alloc] initWithNibName:@"FlipsideViewController" bundle:nil] autorelease];
    controller.delegate = self;
    controller.modalTransitionStyle = UIModalTransitionStyleFlipHorizontal;
    [self presentModalViewController:controller animated:YES];
}

- (IBAction)clickRefreshButton:(id)sender {
    // 자동위치 추적모드가 동작중이 아닐때 위치 정보를 가져오기 위함 
    if (!isBackgroundTrace) { 
        self.locationManager.desiredAccuracy = kCLLocationAccuracyKilometer;	
        self.locationManager.distanceFilter = 500;
        [self.locationManager startUpdatingLocation];
    }
    self.curLocation = self.locationManager.location;
    // 기상특보 데이터를 서버로부터 새로고침
    [self startReceiveSpecialReport];
}

- (IBAction)clickAlertButton:(id)sender {
    [self toggleLocationMonitoring];
    NSString *msg = @"기상특보 알림이 종료 되었습니다.";
    if (isBackgroundTrace) {
        msg = @"기상특보 알림이 시작되었습니다.";
    }
    UIAlertView *alert = [[UIAlertView alloc] 
                          initWithTitle:@"기상특보 알림 설정" 
                          message:msg 
                          delegate:nil 
                          cancelButtonTitle:@"닫기" 
                          otherButtonTitles:nil];
    [alert show];
    [alert release];
}

@end
